@@ -14,6 +14,7 @@
 
 # This class contains the basic functionality needed to run any interpreter
 # or an interpreter-based tool.
+from __future__ import annotations
 
 from .common import CMakeException
 from .generator import parse_generator_expressions
@@ -211,7 +212,7 @@ class CMakeTraceParser:
             p: {k: strlist_gen(v) for k, v in d.items()}
             for p, d in self.vars_by_file.items()
         }
-        self.explicit_headers = set(Path(parse_generator_expressions(str(x), self)) for x in self.explicit_headers)
+        self.explicit_headers = {Path(parse_generator_expressions(str(x), self)) for x in self.explicit_headers}
         self.cache = {
             k: CMakeCacheEntry(
                 strlist_gen(v.value),
@@ -682,14 +683,14 @@ class CMakeTraceParser:
             if i in ignore:
                 continue
 
-            if i in ['INTERFACE', 'LINK_INTERFACE_LIBRARIES', 'PUBLIC', 'PRIVATE', 'LINK_PUBLIC', 'LINK_PRIVATE']:
+            if i in {'INTERFACE', 'LINK_INTERFACE_LIBRARIES', 'PUBLIC', 'PRIVATE', 'LINK_PUBLIC', 'LINK_PRIVATE'}:
                 mode = i
                 continue
 
-            if mode in ['INTERFACE', 'LINK_INTERFACE_LIBRARIES', 'PUBLIC', 'LINK_PUBLIC']:
+            if mode in {'INTERFACE', 'LINK_INTERFACE_LIBRARIES', 'PUBLIC', 'LINK_PUBLIC'}:
                 interface += i.split(';')
 
-            if mode in ['PUBLIC', 'PRIVATE', 'LINK_PRIVATE']:
+            if mode in {'PUBLIC', 'PRIVATE', 'LINK_PRIVATE'}:
                 private += i.split(';')
 
         if paths:
@@ -747,7 +748,7 @@ class CMakeTraceParser:
             func = mo_file_line.group(4)
             args = mo_file_line.group(5)
             argl = args.split(' ')
-            argl = list(map(lambda x: x.strip(), argl))
+            argl = [a.strip() for a in argl]
 
             yield CMakeTraceLine(file, int(line), func, argl)
 
