@@ -1,4 +1,4 @@
-# Copyright 2012-2017 The Meson development team
+# Copyright 2012-2022 The Meson development team
 
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -11,32 +11,42 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
 import os.path
 import re
 import subprocess
 import typing as T
 
+from .. import mesonlib
+from .. import mlog
+from ..arglist import CompilerArgs
+from ..linkers import RSPFileSyntax
 from ..mesonlib import (
-    EnvironmentException, MachineChoice, version_compare, OptionKey,
+    EnvironmentException, version_compare, OptionKey, is_windows
 )
 
-from ..arglist import CompilerArgs
+from . import compilers
 from .compilers import (
     d_dmd_buildtype_args,
     d_gdc_buildtype_args,
     d_ldc_buildtype_args,
     clike_debug_args,
     Compiler,
+    CompileCheckMode,
 )
 from .mixins.gnu import GnuCompiler
+from .mixins.gnu import gnu_common_warning_args
 
 if T.TYPE_CHECKING:
-    from .compilers import Compiler as CompilerMixinBase
-    from ..dependencies import Dependency, ExternalProgram
+    from ..dependencies import Dependency
+    from ..programs import ExternalProgram
     from ..envconfig import MachineInfo
     from ..environment import Environment
     from ..linkers import DynamicLinker
+    from ..mesonlib import MachineChoice
+
+    CompilerMixinBase = Compiler
 else:
     CompilerMixinBase = object
 
@@ -57,15 +67,17 @@ d_feature_args = {'gcc':  {'unittest': '-funittest',
                            }
                   }  # type: T.Dict[str, T.Dict[str, str]]
 
-ldc_optimization_args = {'0': [],
+ldc_optimization_args = {'plain': [],
+                         '0': [],
                          'g': [],
                          '1': ['-O1'],
                          '2': ['-O2'],
                          '3': ['-O3'],
-                         's': ['-Os'],
+                         's': ['-Oz'],
                          }  # type: T.Dict[str, T.List[str]]
 
-dmd_optimization_args = {'0': [],
+dmd_optimization_args = {'plain': [],
+                         '0': [],
                          'g': [],
                          '1': ['-O'],
                          '2': ['-O'],
@@ -103,6 +115,8 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         return ['-of=' + outputname]
 
     def get_include_args(self, path: str, is_system: bool) -> T.List[str]:
+        if path == "":
+            path = "."
         return ['-I=' + path]
 
     def compute_parameters_with_absolute_paths(self, parameter_list: T.List[str],
@@ -153,6 +167,9 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
     def get_feature_args(self, kwargs: T.Dict[str, T.Any], build_to_src: str) -> T.List[str]:
         # TODO: using a TypeDict here would improve this
         res = []
+        # get_feature_args can be called multiple times for the same target when there is generated source
+        # so we have to copy the kwargs (target.d_features) dict before popping from it
+        kwargs = kwargs.copy()
         if 'unittest' in kwargs:
             unittest = kwargs.pop('unittest')
             unittest_arg = d_feature_args[self.id]['unittest']
@@ -180,10 +197,10 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
                     if int(d) > debug_level:
                         debug_level = int(d)
                 else:
-                    res.append('{0}={1}'.format(debug_arg, d))
+                    res.append(f'{debug_arg}={d}')
 
             if debug_level >= 0:
-                res.append('{0}={1}'.format(debug_arg, debug_level))
+                res.append(f'{debug_arg}={debug_level}')
 
         if 'versions' in kwargs:
             version_level = -1
@@ -204,10 +221,10 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
                     if int(v) > version_level:
                         version_level = int(v)
                 else:
-                    res.append('{0}={1}'.format(version_arg, v))
+                    res.append(f'{version_arg}={v}')
 
             if version_level >= 0:
-                res.append('{0}={1}'.format(version_arg, version_level))
+                res.append(f'{version_arg}={version_level}')
 
         if 'import_dirs' in kwargs:
             import_dirs = kwargs.pop('import_dirs')
@@ -220,13 +237,15 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
             for idir_obj in import_dirs:
                 basedir = idir_obj.get_curdir()
                 for idir in idir_obj.get_incdirs():
+                    bldtreedir = os.path.join(basedir, idir)
                     # Avoid superfluous '/.' at the end of paths when d is '.'
                     if idir not in ('', '.'):
-                        expdir = os.path.join(basedir, idir)
+                        expdir = bldtreedir
                     else:
                         expdir = basedir
                     srctreedir = os.path.join(build_to_src, expdir)
-                    res.append('{0}{1}'.format(import_dir_arg, srctreedir))
+                    res.append(f'{import_dir_arg}{srctreedir}')
+                    res.append(f'{import_dir_arg}{bldtreedir}')
 
         if kwargs:
             raise EnvironmentException('Unknown D compiler feature(s) selected: %s' % ', '.join(kwargs.keys()))
@@ -242,7 +261,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         return self.linker.import_library_args(implibname)
 
     def build_rpath_args(self, env: 'Environment', build_dir: str, from_dir: str,
-                         rpath_paths: str, build_rpath: str,
+                         rpath_paths: T.Tuple[str, ...], build_rpath: str,
                          install_rpath: str) -> T.Tuple[T.List[str], T.Set[bytes]]:
         if self.info.is_windows():
             return ([], set())
@@ -267,19 +286,26 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         return super().build_rpath_args(
             env, build_dir, from_dir, rpath_paths, build_rpath, install_rpath)
 
-    def _translate_args_to_nongnu(self, args: T.List[str]) -> T.List[str]:
-        dcargs = []
+    @classmethod
+    def _translate_args_to_nongnu(cls, args: T.List[str], info: MachineInfo, link_id: str) -> T.List[str]:
         # Translate common arguments to flags the LDC/DMD compilers
         # can understand.
         # The flags might have been added by pkg-config files,
         # and are therefore out of the user's control.
+        dcargs = []
+        # whether we hit a linker argument that expect another arg
+        # see the comment in the "-L" section
+        link_expect_arg = False
+        link_flags_with_arg = [
+            '-rpath', '-rpath-link', '-soname', '-compatibility_version', '-current_version',
+        ]
         for arg in args:
             # Translate OS specific arguments first.
             osargs = []  # type: T.List[str]
-            if self.info.is_windows():
-                osargs = self.translate_arg_to_windows(arg)
-            elif self.info.is_darwin():
-                osargs = self._translate_arg_to_osx(arg)
+            if info.is_windows():
+                osargs = cls.translate_arg_to_windows(arg)
+            elif info.is_darwin():
+                osargs = cls._translate_arg_to_osx(arg)
             if osargs:
                 dcargs.extend(osargs)
                 continue
@@ -289,7 +315,9 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
                 continue
             if arg.startswith('-fstack-protector'):
                 continue
-            if arg.startswith('-D'):
+            if arg.startswith('-D') and not (arg == '-D' or arg.startswith(('-Dd', '-Df'))):
+                # ignore all '-D*' flags (like '-D_THREAD_SAFE')
+                # unless they are related to documentation
                 continue
             if arg.startswith('-Wl,'):
                 # Translate linker arguments here.
@@ -327,23 +355,51 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
                 else:
                     dcargs.append('-I' + arg[10:])
                 continue
-            elif arg.startswith('-L/') or arg.startswith('-L./'):
-                # we need to handle cases where -L is set by e.g. a pkg-config
-                # setting to select a linker search path. We can however not
-                # unconditionally prefix '-L' with '-L' because the user might
-                # have set this flag too to do what it is intended to for this
-                # compiler (pass flag through to the linker)
-                # Hence, we guess here whether the flag was intended to pass
-                # a linker search path.
+            elif arg.startswith('-L'):
+                # The D linker expect library search paths in the form of -L=-L/path (the '=' is optional).
+                #
+                # This function receives a mix of arguments already prepended
+                # with -L for the D linker driver and other linker arguments.
+                # The arguments starting with -L can be:
+                #  - library search path (with or without a second -L)
+                #     - it can come from pkg-config (a single -L)
+                #     - or from the user passing linker flags (-L-L would be expected)
+                #  - arguments like "-L=-rpath" that expect a second argument (also prepended with -L)
+                #  - arguments like "-L=@rpath/xxx" without a second argument (on Apple platform)
+                #  - arguments like "-L=/SUBSYSTEM:CONSOLE (for Windows linker)
+                #
+                # The logic that follows tries to detect all these cases (some may be missing)
+                # in order to prepend a -L only for the library search paths with a single -L
+
+                if arg.startswith('-L='):
+                    suffix = arg[3:]
+                else:
+                    suffix = arg[2:]
+
+                if link_expect_arg:
+                    # flags like rpath and soname expect a path or filename respectively,
+                    # we must not alter it (i.e. prefixing with -L for a lib search path)
+                    dcargs.append(arg)
+                    link_expect_arg = False
+                    continue
+
+                if suffix in link_flags_with_arg:
+                    link_expect_arg = True
+
+                if suffix.startswith('-') or suffix.startswith('@'):
+                    # this is not search path
+                    dcargs.append(arg)
+                    continue
+
+                # linker flag such as -L=/DEBUG must pass through
+                if info.is_windows() and link_id == 'link' and suffix.startswith('/'):
+                    dcargs.append(arg)
+                    continue
 
                 # Make sure static library files are passed properly to the linker.
                 if arg.endswith('.a') or arg.endswith('.lib'):
-                    if arg.startswith('-L='):
-                        farg = arg[3:]
-                    else:
-                        farg = arg[2:]
-                    if len(farg) > 0 and not farg.startswith('-'):
-                        dcargs.append('-L=' + farg)
+                    if len(suffix) > 0 and not suffix.startswith('-'):
+                        dcargs.append('-L=' + suffix)
                         continue
 
                 dcargs.append('-L=' + arg)
@@ -390,6 +446,10 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
             args.append('-L=' + arg)
         return args
 
+    @classmethod
+    def _unix_args_to_native(cls, args: T.List[str], info: MachineInfo, link_id: str = '') -> T.List[str]:
+        return cls._translate_args_to_nongnu(args, info, link_id)
+
     def get_debug_args(self, is_debug: bool) -> T.List[str]:
         ddebug_args = []
         if is_debug:
@@ -403,7 +463,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
 
         if crt_val in self.mscrt_args:
             return self.mscrt_args[crt_val]
-        assert(crt_val in ['from_buildtype', 'static_from_buildtype'])
+        assert crt_val in {'from_buildtype', 'static_from_buildtype'}
 
         dbg = 'mdd'
         rel = 'md'
@@ -423,15 +483,14 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         elif buildtype == 'minsize':
             return self.mscrt_args[rel]
         else:
-            assert(buildtype == 'custom')
+            assert buildtype == 'custom'
             raise EnvironmentException('Requested C runtime based on buildtype, but buildtype is "custom".')
 
     def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
                         suffix: str, soversion: str,
-                        darwin_versions: T.Tuple[str, str],
-                        is_shared_module: bool) -> T.List[str]:
+                        darwin_versions: T.Tuple[str, str]) -> T.List[str]:
         sargs = super().get_soname_args(env, prefix, shlib_name, suffix,
-                                        soversion, darwin_versions, is_shared_module)
+                                        soversion, darwin_versions)
 
         # LDC and DMD actually do use a linker, but they proxy all of that with
         # their own arguments
@@ -487,7 +546,7 @@ class DCompiler(Compiler):
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None,
                  is_cross: bool = False):
-        super().__init__(exelist, version, for_machine, info, linker=linker,
+        super().__init__([], exelist, version, for_machine, info, linker=linker,
                          full_version=full_version, is_cross=is_cross)
         self.arch = arch
         self.exe_wrapper = exe_wrapper
@@ -495,7 +554,7 @@ class DCompiler(Compiler):
     def sanity_check(self, work_dir: str, environment: 'Environment') -> None:
         source_name = os.path.join(work_dir, 'sanity.d')
         output_name = os.path.join(work_dir, 'dtest')
-        with open(source_name, 'w') as ofile:
+        with open(source_name, 'w', encoding='utf-8') as ofile:
             ofile.write('''void main() { }''')
         pc = subprocess.Popen(self.exelist + self.get_output_args(output_name) + self._get_target_arch_args() + [source_name], cwd=work_dir)
         pc.wait()
@@ -525,6 +584,9 @@ class DCompiler(Compiler):
     def get_feature_args(self, kwargs: T.Dict[str, T.Any], build_to_src: str) -> T.List[str]:
         # TODO: using a TypeDict here would improve this
         res = []
+        # get_feature_args can be called multiple times for the same target when there is generated source
+        # so we have to copy the kwargs (target.d_features) dict before popping from it
+        kwargs = kwargs.copy()
         if 'unittest' in kwargs:
             unittest = kwargs.pop('unittest')
             unittest_arg = d_feature_args[self.id]['unittest']
@@ -552,10 +614,10 @@ class DCompiler(Compiler):
                     if int(d) > debug_level:
                         debug_level = int(d)
                 else:
-                    res.append('{0}={1}'.format(debug_arg, d))
+                    res.append(f'{debug_arg}={d}')
 
             if debug_level >= 0:
-                res.append('{0}={1}'.format(debug_arg, debug_level))
+                res.append(f'{debug_arg}={debug_level}')
 
         if 'versions' in kwargs:
             version_level = -1
@@ -576,10 +638,10 @@ class DCompiler(Compiler):
                     if int(v) > version_level:
                         version_level = int(v)
                 else:
-                    res.append('{0}={1}'.format(version_arg, v))
+                    res.append(f'{version_arg}={v}')
 
             if version_level >= 0:
-                res.append('{0}={1}'.format(version_arg, version_level))
+                res.append(f'{version_arg}={version_level}')
 
         if 'import_dirs' in kwargs:
             import_dirs = kwargs.pop('import_dirs')
@@ -592,13 +654,15 @@ class DCompiler(Compiler):
             for idir_obj in import_dirs:
                 basedir = idir_obj.get_curdir()
                 for idir in idir_obj.get_incdirs():
+                    bldtreedir = os.path.join(basedir, idir)
                     # Avoid superfluous '/.' at the end of paths when d is '.'
                     if idir not in ('', '.'):
-                        expdir = os.path.join(basedir, idir)
+                        expdir = bldtreedir
                     else:
                         expdir = basedir
                     srctreedir = os.path.join(build_to_src, expdir)
-                    res.append('{0}{1}'.format(import_dir_arg, srctreedir))
+                    res.append(f'{import_dir_arg}{srctreedir}')
+                    res.append(f'{import_dir_arg}{bldtreedir}')
 
         if kwargs:
             raise EnvironmentException('Unknown D compiler feature(s) selected: %s' % ', '.join(kwargs.keys()))
@@ -631,11 +695,104 @@ class DCompiler(Compiler):
     def get_crt_link_args(self, crt_val: str, buildtype: str) -> T.List[str]:
         return []
 
+    def _get_compile_extra_args(self, extra_args: T.Union[T.List[str], T.Callable[[CompileCheckMode], T.List[str]], None] = None) -> T.List[str]:
+        args = self._get_target_arch_args()
+        if extra_args:
+            if callable(extra_args):
+                extra_args = extra_args(CompileCheckMode.COMPILE)
+            if isinstance(extra_args, list):
+                args.extend(extra_args)
+            elif isinstance(extra_args, str):
+                args.append(extra_args)
+        return args
+
+    def run(self, code: 'mesonlib.FileOrString', env: 'Environment', *,
+            extra_args: T.Union[T.List[str], T.Callable[[CompileCheckMode], T.List[str]], None] = None,
+            dependencies: T.Optional[T.List['Dependency']] = None) -> compilers.RunResult:
+        need_exe_wrapper = env.need_exe_wrapper(self.for_machine)
+        if need_exe_wrapper and self.exe_wrapper is None:
+            raise compilers.CrossNoRunException('Can not run test applications in this cross environment.')
+        extra_args = self._get_compile_extra_args(extra_args)
+        with self._build_wrapper(code, env, extra_args, dependencies, mode='link', want_output=True) as p:
+            if p.returncode != 0:
+                mlog.debug(f'Could not compile test file {p.input_name}: {p.returncode}\n')
+                return compilers.RunResult(False)
+            if need_exe_wrapper:
+                cmdlist = self.exe_wrapper.get_command() + [p.output_name]
+            else:
+                cmdlist = [p.output_name]
+            try:
+                pe, so, se = mesonlib.Popen_safe(cmdlist)
+            except Exception as e:
+                mlog.debug(f'Could not run: {cmdlist} (error: {e})\n')
+                return compilers.RunResult(False)
+
+        mlog.debug('Program stdout:\n')
+        mlog.debug(so)
+        mlog.debug('Program stderr:\n')
+        mlog.debug(se)
+        return compilers.RunResult(True, pe.returncode, so, se)
+
+    def sizeof(self, typename: str, prefix: str, env: 'Environment', *,
+               extra_args: T.Union[None, T.List[str], T.Callable[[CompileCheckMode], T.List[str]]] = None,
+               dependencies: T.Optional[T.List['Dependency']] = None) -> int:
+        if extra_args is None:
+            extra_args = []
+        t = f'''
+        import std.stdio : writeln;
+        {prefix}
+        void main() {{
+            writeln(({typename}).sizeof);
+        }}
+        '''
+        res = self.run(t, env, extra_args=extra_args,
+                       dependencies=dependencies)
+        if not res.compiled:
+            return -1
+        if res.returncode != 0:
+            raise mesonlib.EnvironmentException('Could not run sizeof test binary.')
+        return int(res.stdout)
+
+    def alignment(self, typename: str, prefix: str, env: 'Environment', *,
+                  extra_args: T.Optional[T.List[str]] = None,
+                  dependencies: T.Optional[T.List['Dependency']] = None) -> int:
+        if extra_args is None:
+            extra_args = []
+        t = f'''
+        import std.stdio : writeln;
+        {prefix}
+        void main() {{
+            writeln(({typename}).alignof);
+        }}
+        '''
+        res = self.run(t, env, extra_args=extra_args,
+                       dependencies=dependencies)
+        if not res.compiled:
+            raise mesonlib.EnvironmentException('Could not compile alignment test.')
+        if res.returncode != 0:
+            raise mesonlib.EnvironmentException('Could not run alignment test binary.')
+        align = int(res.stdout)
+        if align == 0:
+            raise mesonlib.EnvironmentException(f'Could not determine alignment of {typename}. Sorry. You might want to file a bug.')
+        return align
+
+    def has_header(self, hname: str, prefix: str, env: 'Environment', *,
+                   extra_args: T.Union[None, T.List[str], T.Callable[['CompileCheckMode'], T.List[str]]] = None,
+                   dependencies: T.Optional[T.List['Dependency']] = None,
+                   disable_cache: bool = False) -> T.Tuple[bool, bool]:
+
+        extra_args = self._get_compile_extra_args(extra_args)
+        code = f'''{prefix}
+        import {hname};
+        '''
+        return self.compiles(code, env, extra_args=extra_args,
+                             dependencies=dependencies, mode='compile', disable_cache=disable_cache)
 
 class GnuDCompiler(GnuCompiler, DCompiler):
 
     # we mostly want DCompiler, but that gives us the Compiler.LINKER_PREFIX instead
     LINKER_PREFIX = GnuCompiler.LINKER_PREFIX
+    id = 'gcc'
 
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
@@ -647,12 +804,14 @@ class GnuDCompiler(GnuCompiler, DCompiler):
                            exe_wrapper=exe_wrapper, linker=linker,
                            full_version=full_version, is_cross=is_cross)
         GnuCompiler.__init__(self, {})
-        self.id = 'gcc'
         default_warn_args = ['-Wall', '-Wdeprecated']
         self.warn_args = {'0': [],
                           '1': default_warn_args,
                           '2': default_warn_args + ['-Wextra'],
-                          '3': default_warn_args + ['-Wextra', '-Wpedantic']}
+                          '3': default_warn_args + ['-Wextra', '-Wpedantic'],
+                          'everything': (default_warn_args + ['-Wextra', '-Wpedantic'] +
+                                         self.supported_warn_args(gnu_common_warning_args))}
+
         self.base_options = {
             OptionKey(o) for o in [
              'b_colorout', 'b_sanitize', 'b_staticpic', 'b_vscrt',
@@ -715,6 +874,8 @@ def find_ldc_dmd_frontend_version(version_output: T.Optional[str]) -> T.Optional
 
 class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
 
+    id = 'llvm'
+
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
                  exe_wrapper: T.Optional['ExternalProgram'] = None,
@@ -725,7 +886,6 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
                            exe_wrapper=exe_wrapper, linker=linker,
                            full_version=full_version, is_cross=is_cross)
         DmdLikeCompilerMixin.__init__(self, dmd_frontend_version=find_ldc_dmd_frontend_version(version_output))
-        self.id = 'llvm'
         self.base_options = {OptionKey(o) for o in ['b_coverage', 'b_colorout', 'b_vscrt', 'b_ndebug']}
 
     def get_colorout_args(self, colortype: str) -> T.List[str]:
@@ -752,14 +912,14 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
         return self._get_crt_args(crt_val, buildtype)
 
     def unix_args_to_native(self, args: T.List[str]) -> T.List[str]:
-        return self._translate_args_to_nongnu(args)
+        return self._unix_args_to_native(args, self.info, self.linker.id)
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
         return ldc_optimization_args[optimization_level]
 
     @classmethod
-    def use_linker_args(cls, linker: str) -> T.List[str]:
-        return ['-linker={}'.format(linker)]
+    def use_linker_args(cls, linker: str, version: str) -> T.List[str]:
+        return [f'-linker={linker}']
 
     def get_linker_always_args(self) -> T.List[str]:
         args = super().get_linker_always_args()
@@ -770,9 +930,17 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
     def get_disable_assert_args(self) -> T.List[str]:
         return ['--release']
 
+    def rsp_file_syntax(self) -> RSPFileSyntax:
+        # We use `mesonlib.is_windows` here because we want to know what the
+        # build machine is, not the host machine. This really means we would
+        # have the Environment not the MachineInfo in the compiler.
+        return RSPFileSyntax.MSVC if is_windows() else RSPFileSyntax.GCC
+
 
 class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
 
+    id = 'dmd'
+
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
                  exe_wrapper: T.Optional['ExternalProgram'] = None,
@@ -783,7 +951,6 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
                            exe_wrapper=exe_wrapper, linker=linker,
                            full_version=full_version, is_cross=is_cross)
         DmdLikeCompilerMixin.__init__(self, version)
-        self.id = 'dmd'
         self.base_options = {OptionKey(o) for o in ['b_coverage', 'b_colorout', 'b_vscrt', 'b_ndebug']}
 
     def get_colorout_args(self, colortype: str) -> T.List[str]:
@@ -834,7 +1001,7 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
         return self._get_crt_args(crt_val, buildtype)
 
     def unix_args_to_native(self, args: T.List[str]) -> T.List[str]:
-        return self._translate_args_to_nongnu(args)
+        return self._unix_args_to_native(args, self.info, self.linker.id)
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
         return dmd_optimization_args[optimization_level]
@@ -850,3 +1017,6 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
 
     def get_disable_assert_args(self) -> T.List[str]:
         return ['-release']
+
+    def rsp_file_syntax(self) -> RSPFileSyntax:
+        return RSPFileSyntax.MSVC
