@@ -45,6 +45,7 @@ from .linkers import (
     GnuGoldDynamicLinker,
     GnuBFDDynamicLinker,
     LLVMDynamicLinker,
+    MoldDynamicLinker,
     WASMDynamicLinker,
     CcrxDynamicLinker,
     Xc16DynamicLinker,
@@ -106,6 +107,7 @@ __all__ = [
     'GnuGoldDynamicLinker',
     'GnuBFDDynamicLinker',
     'LLVMDynamicLinker',
+    'MoldDynamicLinker',
     'WASMDynamicLinker',
     'CcrxDynamicLinker',
     'Xc16DynamicLinker',
