@@ -91,13 +91,20 @@ mod_kwargs -= {'name_prefix', 'name_suffix'}
 class _PythonDependencyBase(_Base):
 
     def __init__(self, python_holder: 'PythonInstallation', embed: bool):
-        self.name = 'python'  # override the name from the "real" dependency lookup
         self.embed = embed
         self.version: str = python_holder.version
         self.platform = python_holder.platform
         self.variables = python_holder.variables
         self.paths = python_holder.paths
-        self.link_libpython = python_holder.link_libpython
+        # The "-embed" version of python.pc / python-config was introduced in 3.8,
+        # and distutils extension linking was changed to be considered a non embed
+        # usage. Before then, this dependency always uses the embed=True handling
+        # because that is the only one that exists.
+        #
+        # On macOS and some Linux distros (Debian) distutils doesn't link extensions
+        # against libpython, even on 3.7 and below. We call into distutils and
+        # mirror its behavior. See https://github.com/mesonbuild/meson/issues/4117
+        self.link_libpython = python_holder.link_libpython or embed
         self.info: T.Optional[T.Dict[str, str]] = None
         if mesonlib.version_compare(self.version, '>= 3.0'):
             self.major_version = 3
@@ -121,6 +128,10 @@ class PythonPkgConfigDependency(PkgConfigDependency, _PythonDependencyBase):
         if libpc and not self.is_found:
             mlog.debug(f'"python-{self.version}" could not be found in LIBPC, this is likely due to a relocated python installation')
 
+        # pkg-config files are usually accurate starting with python 3.8
+        if not self.link_libpython and mesonlib.version_compare(self.version, '< 3.8'):
+            self.link_args = []
+
 
 class PythonFrameworkDependency(ExtraFrameworkDependency, _PythonDependencyBase):
 
@@ -142,6 +153,13 @@ class PythonSystemDependency(SystemDependency, _PythonDependencyBase):
         else:
             self._find_libpy(installation, environment)
 
+        if not self.link_libpython:
+            # match pkg-config behavior
+            self.link_args = []
+
+        if not self.clib_compiler.has_header('Python.h', '', environment, extra_args=self.compile_args):
+            self.is_found = False
+
     def _find_libpy(self, python_holder: 'PythonInstallation', environment: 'Environment') -> None:
         if python_holder.is_pypy:
             if self.major_version == 3:
@@ -183,7 +201,7 @@ class PythonSystemDependency(SystemDependency, _PythonDependencyBase):
                 return None
         elif self.platform == 'win32':
             return '32'
-        elif self.platform in ('win64', 'win-amd64'):
+        elif self.platform in {'win64', 'win-amd64'}:
             return '64'
         mlog.log(f'Unknown Windows Python platform {self.platform!r}')
         return None
@@ -263,7 +281,8 @@ class PythonSystemDependency(SystemDependency, _PythonDependencyBase):
         self.compile_args += ['-I' + path for path in inc_paths if path]
 
         # https://sourceforge.net/p/mingw-w64/mailman/message/30504611/
-        if pyarch == '64' and self.major_version == 2:
+        # https://github.com/python/cpython/pull/100137
+        if pyarch == '64' and mesonlib.version_compare(self.version, '<3.12'):
             self.compile_args += ['-DMS_WIN64']
 
         self.is_found = True
@@ -372,6 +391,15 @@ def links_against_libpython():
 variables = sysconfig.get_config_vars()
 variables.update({'base_prefix': getattr(sys, 'base_prefix', sys.prefix)})
 
+if sys.version_info < (3, 0):
+    suffix = variables.get('SO')
+elif sys.version_info < (3, 8, 7):
+    # https://bugs.python.org/issue?@action=redirect&bpo=39825
+    from distutils.sysconfig import get_config_var
+    suffix = get_config_var('EXT_SUFFIX')
+else:
+    suffix = variables.get('EXT_SUFFIX')
+
 print(json.dumps({
   'variables': variables,
   'paths': paths,
@@ -382,6 +410,7 @@ print(json.dumps({
   'is_pypy': '__pypy__' in sys.builtin_module_names,
   'is_venv': sys.prefix != variables['base_prefix'],
   'link_libpython': links_against_libpython(),
+  'suffix': suffix,
 }))
 '''
 
@@ -440,8 +469,6 @@ class PythonExternalProgram(ExternalProgram):
             mlog.debug(stderr)
 
         if info is not None and self._check_version(info['version']):
-            variables = info['variables']
-            info['suffix'] = variables.get('EXT_SUFFIX') or variables.get('SO') or variables.get('.so')
             self.info = T.cast('PythonIntrospectionDict', info)
             self.platlib = self._get_path(state, 'platlib')
             self.purelib = self._get_path(state, 'purelib')
@@ -521,17 +548,8 @@ class PythonInstallation(ExternalProgramHolder):
 
             kwargs['install_dir'] = self._get_install_dir_impl(False, subdir)
 
-        new_deps = []
-        has_pydep = False
-        for dep in mesonlib.extract_as_list(kwargs, 'dependencies'):
-            if isinstance(dep, _PythonDependencyBase):
-                has_pydep = True
-                # On macOS and some Linux distros (Debian) distutils doesn't link
-                # extensions against libpython. We call into distutils and mirror its
-                # behavior. See https://github.com/mesonbuild/meson/issues/4117
-                if not self.link_libpython:
-                    dep = dep.get_partial_dependency(compile_args=True)
-            new_deps.append(dep)
+        new_deps = mesonlib.extract_as_list(kwargs, 'dependencies')
+        has_pydep = any(isinstance(dep, _PythonDependencyBase) for dep in new_deps)
         if not has_pydep:
             pydep = self._dependency_method_impl({})
             if not pydep.found():
@@ -602,7 +620,7 @@ class PythonInstallation(ExternalProgramHolder):
     )
     def install_sources_method(self, args: T.Tuple[T.List[T.Union[str, mesonlib.File]]],
                                kwargs: 'PyInstallKw') -> 'Data':
-        tag = kwargs['install_tag'] or 'runtime'
+        tag = kwargs['install_tag'] or 'python-runtime'
         pure = kwargs['pure'] if kwargs['pure'] is not None else self.pure
         install_dir = self._get_install_dir_impl(pure, kwargs['subdir'])
         return self.interpreter.install_data_impl(
@@ -717,7 +735,7 @@ class PythonModule(ExtensionModule):
             # on various platforms, let's not give up just yet, if an executable
             # named python is available and has a compatible version, let's use
             # it
-            if not python.found() and name_or_path in ['python2', 'python3']:
+            if not python.found() and name_or_path in {'python2', 'python3'}:
                 python = PythonExternalProgram('python')
 
         if python.found():
