@@ -15,9 +15,10 @@
 import subprocess
 import json
 import os
+import shutil
 import unittest
-from distutils.dir_util import copy_tree
 
+from mesonbuild.mesonlib import windows_proof_rmtree
 from .baseplatformtests import BasePlatformTests
 
 class RewriterTests(BasePlatformTests):
@@ -26,14 +27,15 @@ class RewriterTests(BasePlatformTests):
         self.maxDiff = None
 
     def prime(self, dirname):
-        copy_tree(os.path.join(self.rewrite_test_dir, dirname), self.builddir)
+        if os.path.exists(self.builddir):
+            windows_proof_rmtree(self.builddir)
+        shutil.copytree(os.path.join(self.rewrite_test_dir, dirname), self.builddir)
 
     def rewrite_raw(self, directory, args):
         if isinstance(args, str):
             args = [args]
         command = self.rewrite_command + ['--verbose', '--skip', '--sourcedir', directory] + args
-        p = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
-                           universal_newlines=True, timeout=60)
+        p = subprocess.run(command, capture_output=True, text=True, timeout=60)
         print('STDOUT:')
         print(p.stdout)
         print('STDERR:')
